@@ -1,25 +1,27 @@
-"""说明 Instructions:
+"""说明 Introduction:
 A simple text editor written in Python.It supports editing text files,
 binary files ,encodings and changing font size.
 When you edit a binary file, the contents of the file are
 displayed as escape sequences.
 You can also find and replace words.
 In addition, code highlighting is supported when editing Python code files,like IDLE.
+What's more, dragging and dropping files into the editor window is now supported.
 
 一款使用tkinter编写的文本编辑器, 支持编辑文本文件、二进制文件、改变字体大小。
 支持ansi、gbk、utf-8等编码。编辑二进制文件时, 文件内容以转义序列形式显示。
-支持查找、替换、改变字体大小; 且支持撤销、重做; 支持将文件拖入窗口
+支持查找、替换、改变字体大小; 且支持撤销、重做; 支持将文件拖放入窗口。
 编辑python代码文件时, 支持代码高亮显示, 类似IDLE。
 
 作者:qfcy (七分诚意)
-版本:1.2.8.04
+版本:1.2.8.05
 """
-import sys,os,time
+import sys,os,time,pickle
 from tkinter import *
 from tkinter.scrolledtext import ScrolledText
 import tkinter.ttk as ttk
 import tkinter.messagebox as msgbox
 import tkinter.filedialog as dialog
+import tkinter.simpledialog as simpledialog
 
 try:
     from idlelib.colorizer import ColorDelegator
@@ -31,7 +33,7 @@ except ImportError:windnd=None
 
 __email__="3416445406@qq.com"
 __author__="七分诚意 qq:3076711200 邮箱:%s"%__email__
-__version__="1.2.8.04"
+__version__="1.2.8.05"
 
 def view_hex(bytes):
     result=''
@@ -111,7 +113,7 @@ class SearchDialog(Toplevel):
         .pack(side=LEFT)
         self.use_escape_char=IntVar(self.master)
         self.use_escape_char.set(self.master.isbinary)
-        ttk.Checkbutton(master,text="转义字符",variable=self.use_escape_char)\
+        ttk.Checkbutton(master,text="使用转义字符",variable=self.use_escape_char)\
         .pack(side=LEFT)
 
     def search(self,event=None,mark=True,bell=True):
@@ -126,6 +128,10 @@ class SearchDialog(Toplevel):
         pos=text.search(key,INSERT,END,
                         regexp=self.use_regexpr.get(),
                         nocase=not self.match_case.get())
+        if not pos:
+            pos=text.search(key,'1.0',END, # 尝试从开头循环查找
+                        regexp=self.use_regexpr.get(),
+                        nocase=not self.match_case.get())
         if pos:
             newpos="%s+%dc"%(pos,len(key))
             text.mark_set(INSERT,newpos)
@@ -190,7 +196,10 @@ class ReplaceDialog(SearchDialog):
 
     def replace_all(self):
         self.master.contents.mark_set("insert","1.0")
-        while self.replace(bell=False)!=-1:pass
+        flag=False
+        while self.replace(bell=False)!=-1:
+            flag=True
+        if not flag:bell_()
      
 class Editor(Tk):
     TITLE="PyNotepad"
@@ -201,6 +210,8 @@ class Editor(Tk):
     NORMAL_FONT='宋体'
     NORMAL_FONTSIZE=11
     FILETYPES=[("所有文件","*.*")]
+    CONFIGFILE=os.getenv("userprofile")+"\.pynotepad.pkl"
+
     windows=[]
     def __init__(self,filename=""):
         super().__init__()
@@ -213,11 +224,10 @@ class Editor(Tk):
 
         self.isbinary=self.file_changed=False
         self.colorobj=self._codefilter=None
-        self.coding=StringVar(self)
-        self.coding.set(self.NORMAL_CODING)
         Editor.windows.append(self)
 
         self.load_icon()
+        self.loadconfig()
         self.create_widgets()
         self.wm_deiconify();self.update() # wm_deiconfy恢复被隐藏的窗口
         if windnd:windnd.hook_dropfiles(self,func=self.onfiledrag);self.drag_files=[]
@@ -249,6 +259,8 @@ class Editor(Tk):
         ttk.Button(frame,text='保存', command=self.save,width=7).pack(side=LEFT)
 
         Label(frame,text="编码:").pack(side=LEFT)
+        self.coding=StringVar(self)
+        self.coding.set(self.NORMAL_CODING)
         coding=ttk.Combobox(frame,textvariable=self.coding)
         def tip(event):
             self.msg['text']='重新打开或保存即可生效'
@@ -261,7 +273,7 @@ class Editor(Tk):
         self.msg.pack(side=LEFT)
 
         self.contents=ScrolledText(self,undo=True,width=75,height=24,
-                                   font=(self.NORMAL_FONT,self.NORMAL_FONTSIZE,"normal"))
+                        font=(self.NORMAL_FONT,self.NORMAL_FONTSIZE,"normal"))
         self.contents.pack(expand=True,fill=BOTH)
         self.contents.bind("<Key>",self.text_change)
         self.contents.bind("<B1-ButtonRelease>",self.update_status)
@@ -319,6 +331,9 @@ class Editor(Tk):
         view.add_checkbutton(label="自动换行", command=self.set_wrap,
                              variable=self.is_autowrap)
         fontsize=Menu(self.contents,tearoff=False)
+        fontsize.add_command(label="选择字体",
+                             command=self.choose_font)
+        fontsize.add_separator()
         fontsize.add_command(label="增大字体   ",accelerator='Ctrl+ "+"',
                              command=self.increase_font)
         fontsize.add_command(label="减小字体   ",accelerator='Ctrl+ "-"',
@@ -366,15 +381,22 @@ class Editor(Tk):
 
     def set_fontsize(self,index):
         newsize=self.FONTSIZES[index]
-        self.contents["font"]=(self.NORMAL_FONT,newsize,"normal")
+        fontname=self.contents["font"].split(' ')[0]
+        self.contents["font"]=(fontname,newsize,"normal")
+    def choose_font(self):
+        oldfont=self.contents["font"].split(' ')[0]
+        font=simpledialog.askstring(
+                '', '输入字体名称: (例如: 楷体)', initialvalue = oldfont)
+        if font is not None:
+            self.contents["font"]=[font] + self.contents["font"].split(' ')[1:]
     def increase_font(self):
         # 增大字体
-        fontsize=int(self.contents["font"].split()[1])
+        fontsize=int(self.contents["font"].split(' ')[1])
         index=self.FONTSIZES.index(fontsize)+1
         if 0<=index<len(self.FONTSIZES): self.set_fontsize(index)
     def decrease_font(self):
         # 减小字体
-        fontsize=int(self.contents["font"].split()[1])
+        fontsize=int(self.contents["font"].split(' ')[1])
         index=self.FONTSIZES.index(fontsize)-1
         if 0<=index<len(self.FONTSIZES): self.set_fontsize(index)
     def set_wrap(self):
@@ -400,7 +422,7 @@ class Editor(Tk):
             elif key=='minus':#Ctrl+ "-" 减小字体
                 self.decrease_font()
     def focus(self,event):
-        #当窗口获得或失去焦点时,调用此函数, 用于使对话框置于主窗体前
+        # 当窗口获得或失去焦点时,调用此函数, 用于使对话框置于主窗体前
         for window in SearchDialog.instances + ReplaceDialog.instances:
             if window.master is self:
                 if event.type==EventType.FocusIn:
@@ -435,7 +457,7 @@ class Editor(Tk):
                 self.charmap.delete("1.0",END)
                 self.charmap.insert(INSERT,view_hex(data))
                 self.status["text"]="选区长度: %d (Bytes)"%len(data)
-            except (TclError,SyntaxError): #未选取内容, 或格式不正确
+            except (TclError,SyntaxError): #忽略未选取内容, 或格式不正确
                 self.bin_data.delete("1.0",END)
                 self.charmap.delete("1.0",END)
                 self.update_offset()
@@ -483,7 +505,7 @@ class Editor(Tk):
                                 filetypes=self.FILETYPES)
         self.load(filename,binary=True)
     def load(self,filename,binary=False):
-        # 加载文件(主要部分)
+        # 加载文件
         if not filename.strip():
             return
         self.isbinary=binary
@@ -508,7 +530,8 @@ class Editor(Tk):
             self.contents.focus_force()
         except Exception as err:handle(err,parent=self)
     def _load_data(self,filename):
-        f=open(filename,"rb")#打开文件
+        # 从文件加载数据
+        f=open(filename,"rb")
         if self.isbinary:
             data=to_escape_str(f.read())
             return data
@@ -567,6 +590,7 @@ class Editor(Tk):
                 my_ret=0;quit=False  # 0表示cancel
         if quit:
             Editor.windows.remove(self)
+            self.saveconfig()
             self.destroy()
         return my_ret
     def save(self):
@@ -602,6 +626,21 @@ class Editor(Tk):
 
     def about(self):
         msgbox.showinfo("关于",__doc__+"\n作者: "+__author__,parent=self)
+    def loadconfig(self):
+        try:
+            with open(self.CONFIGFILE,'rb') as f:
+                cfg=pickle.load(f)
+                for key in cfg:
+                    setattr(Editor,key,cfg[key])
+        except OSError:
+            pass
+    def saveconfig(self):
+        font=self.contents['font'].split(' ')
+        cfg={'NORMAL_CODING':self.coding.get(),
+             'NORMAL_FONT': font[0],
+             'NORMAL_FONTSIZE': int(font[1])}
+        with open(self.CONFIGFILE,'wb') as f:
+            pickle.dump(cfg,f)
     def onfiledrag(self,files):
         self.drag_files=files
         self.after(50,self.onfiledrag2)
